@@ -41,7 +41,6 @@ const pkg = require('../package');
 const secureContext = require('../tls/secure-context');
 const srv = require('../topology/dns-srv');
 const system = require('../system');
-const tlsVersions = require('../tls/versions');
 const util = require('util');
 const warnings = require('../constants/warnings');
 const { isValidBoolean, isValidInteger, isValidPlainObject } = require('../validator');
@@ -66,7 +65,7 @@ const { isValidBoolean, isValidInteger, isValidPlainObject } = require('../valid
  * @prop {boolean} [enabled=true] - enables or disables TLS
  * @prop {string} [ca] - path of a file containing a certificate authority chain used to verify the server certificate
  * @prop {string} [crl] - path of a file containing a certificate revocation list used, alongside a certificate authority, to verify the server certificate
- * @prop {string[]} [versions=TLSv1, TLSv1.1, TLSv1.2, TLSv1.3] - restrict the list of allowed TLS versions (TLSv1, TLSv1.1, TLSv1.2, TLSv1.3)
+ * @prop {string[]} [versions=TLSv1.2, TLSv1.3] - restrict the list of allowed TLS versions (TLSv1.2, TLSv1.3)
  * @prop {string[]} [ciphershuites] - list of ciphersuites to allow (IANA syntax)
  */
 
@@ -764,17 +763,6 @@ function Connection ({ auth, connectionAttributes = {}, connectTimeout = 10000,
                 const nodeSocket = TLS.connect(Object.assign({}, { socket: state.client.getConnection() }, secureContext.create(secureContextOptions)));
 
                 nodeSocket.once('secureConnect', () => {
-                    const tlsVersion = nodeSocket.getProtocol();
-                    // Check if the connection is using a deprecated TLS version.
-                    if (tlsVersions.deprecated().indexOf(tlsVersion) > -1) {
-                        // If it is, a warning should be generated and logged
-                        // or printed to stdout.
-                        logger('connection:tls').warning('version', util.format(warnings.MESSAGES.WARN_DEPRECATED_TLS_VERSION, tlsVersion), {
-                            type: warnings.TYPES.DEPRECATION,
-                            code: warnings.CODES.DEPRECATION
-                        });
-                    }
-
                     state.isSecure = true;
                     // Once a TLS session is established, we need to
                     // update the socket reference.
