@@ -1124,7 +1124,7 @@ describe('X DevAPI Connection', () => {
     });
 
     context('enableTLS()', () => {
-        let Client, log, secureContext, socket, tls, tlsVersions;
+        let Client, secureContext, socket, tls;
 
         beforeEach('create fakes', () => {
             secureContext = td.function();
@@ -1135,9 +1135,7 @@ describe('X DevAPI Connection', () => {
             socket.getProtocol = td.function();
 
             Client = td.replace('../../../lib/Protocol/Client');
-            log = td.replace('../../../lib/logger');
             secureContext = td.replace('../../../lib/tls/secure-context');
-            tlsVersions = td.replace('../../../lib/tls/versions');
             tls = td.replace('tls');
 
             connection = require('../../../lib/DevAPI/Connection');
@@ -1155,8 +1153,6 @@ describe('X DevAPI Connection', () => {
                 return socket;
             });
 
-            td.when(tlsVersions.deprecated()).thenReturn([]);
-
             return con.enableTLS()
                 .then(() => {
                     expect(td.explain(Client.prototype.setConnection).callCount).to.equal(1);
@@ -1166,57 +1162,6 @@ describe('X DevAPI Connection', () => {
                 });
         });
 
-        it('generates a deprecation message if the negotiated TLS version is deprecated', () => {
-            const tlsOptions = { option: 'foo' };
-            const con = connection({ tls: tlsOptions }).setClient(new Client());
-            const warning = td.function();
-
-            td.when(Client.prototype.getConnection()).thenReturn('bar');
-            td.when(secureContext.create({ option: 'foo' })).thenReturn({ option: 'baz' });
-
-            td.when(tls.connect({ socket: 'bar', option: 'baz' })).thenDo(() => {
-                setTimeout(() => socket.emit('secureConnect'));
-                return socket;
-            });
-
-            td.when(socket.getProtocol()).thenReturn('qux');
-            td.when(tlsVersions.deprecated()).thenReturn(['qux', 'quux']);
-            td.when(log('connection:tls')).thenReturn({ warning });
-
-            return con.enableTLS()
-                .then(() => {
-                    expect(td.explain(warning).callCount).to.equal(1);
-                    expect(td.explain(warning).calls[0].args).to.be.an('array').and.have.lengthOf(3);
-                    expect(td.explain(warning).calls[0].args[0]).to.equal('version');
-                    expect(td.explain(warning).calls[0].args[1]).to.equal(util.format(warnings.MESSAGES.WARN_DEPRECATED_TLS_VERSION, 'qux'));
-                    return expect(td.explain(warning).calls[0].args[2]).to.deep.equal({ type: warnings.TYPES.DEPRECATION, code: warnings.CODES.DEPRECATION });
-                });
-        });
-
-        it('does not generate a deprecation message if the negotiated TLS version is not deprecated', () => {
-            const tlsOptions = { option: 'foo' };
-            const con = connection({ tls: tlsOptions }).setClient(new Client());
-            const warning = td.function();
-
-            td.when(Client.prototype.getConnection()).thenReturn('bar');
-            td.when(secureContext.create({ option: 'foo' })).thenReturn({ option: 'baz' });
-
-            td.when(tls.connect({ socket: 'bar', option: 'baz' })).thenDo(() => {
-                setTimeout(() => socket.emit('secureConnect'));
-                return socket;
-            });
-
-            td.when(socket.getProtocol()).thenReturn('qux');
-            td.when(tlsVersions.deprecated()).thenReturn(['quux']);
-            td.when(log('connection:tls')).thenReturn({ warning });
-
-            return con.enableTLS()
-                .then(() => {
-                    expect(td.explain(log).callCount).to.equal(0);
-                    return expect(td.explain(warning).callCount).to.equal(0);
-                });
-        });
-
         it('delegates message handling to the X Protocol client instance', () => {
             const con = connection().setClient(new Client());
 
@@ -1230,8 +1175,6 @@ describe('X DevAPI Connection', () => {
                 return socket;
             });
 
-            td.when(tlsVersions.deprecated()).thenReturn([]);
-
             return con.enableTLS()
                 .then(() => {
                     expect(td.explain(Client.prototype.handleNetworkFragment).callCount).to.equal(2);
@@ -1253,8 +1196,6 @@ describe('X DevAPI Connection', () => {
                 return socket;
             });
 
-            td.when(tlsVersions.deprecated()).thenReturn([]);
-
             return con.enableTLS()
                 .then(() => {
                     return expect(con.getError()).to.deep.equal(error);
