@@ -62,8 +62,18 @@ describe('TLS secure context utilities', () => {
                 secureContext = require('../../../lib/tls/secure-context');
             });
 
-            it('defines the range using the list of supported TLS versions', () => {
-                const versions = ['TLSv1', 'TLSv1.1', 'TLSv1.2', 'TLSv1.3'];
+            it('defines the range using a list of supported TLS versions', () => {
+                const versions = ['TLSv1.2', 'TLSv1.3'];
+
+                td.when(tlsVersions.allowed()).thenReturn(versions);
+                td.when(tlsVersions.supported()).thenReturn(versions.slice(1));
+                td.when(tlsCiphers.defaults()).thenReturn([]);
+
+                expect(secureContext.create()).to.deep.include({ maxVersion: versions[versions.length - 1], minVersion: versions[versions.length - 1] });
+            });
+
+            it('defines the range using the list of provided TLS versions', () => {
+                const versions = ['TLSv1.2', 'TLSv1.3'];
 
                 td.when(tlsVersions.allowed()).thenReturn(versions);
                 td.when(tlsVersions.supported()).thenReturn(versions);
@@ -71,20 +81,6 @@ describe('TLS secure context utilities', () => {
 
                 return expect(secureContext.create()).to.deep.include({ maxVersion: versions[versions.length - 1], minVersion: versions[0] });
             });
-
-            it('defines the range using a list of provided TLS versions', () => {
-                const versions = ['TLSv1', 'TLSv1.1', 'TLSv1.2', 'TLSv1.3'];
-
-                td.when(tlsVersions.allowed()).thenReturn(versions);
-                td.when(tlsVersions.supported()).thenReturn(versions.slice(2));
-                td.when(tlsCiphers.defaults()).thenReturn([]);
-
-                expect(secureContext.create()).to.deep.include({ maxVersion: versions[versions.length - 1], minVersion: versions[2] });
-
-                td.when(tlsVersions.supported()).thenReturn(versions.slice(0, 2));
-
-                return expect(secureContext.create()).to.deep.include({ maxVersion: versions[1], minVersion: versions[0] });
-            });
         });
 
         context('when a range of TLS versions is not supported', () => {
@@ -95,17 +91,13 @@ describe('TLS secure context utilities', () => {
             });
 
             it('defines the latest supported TLS version as the target', () => {
-                const versions = ['TLSv1', 'TLSv1.1', 'TLSv1.2'];
+                const versions = ['TLSv1.2'];
 
                 td.when(tlsVersions.allowed()).thenReturn(versions);
                 td.when(tlsVersions.supported()).thenReturn(versions);
                 td.when(tlsCiphers.defaults()).thenReturn([]);
 
                 expect(secureContext.create()).to.deep.include({ secureProtocol: `${versions[versions.length - 1]}_client_method`.replace('.', '_') });
-
-                td.when(tlsVersions.supported()).thenReturn(versions.slice(0, 2));
-
-                return expect(secureContext.create()).to.deep.include({ secureProtocol: `${versions[1]}_client_method`.replace('.', '_') });
             });
         });
 
@@ -137,7 +129,7 @@ describe('TLS secure context utilities', () => {
             return expect(secureContext.create()).to.deep.include({ rejectUnauthorized: false });
         });
 
-        context('when a path to a CA chain PEM file is provided', () => {
+        context('when a path to a certificate authority chain PEM file is provided', () => {
             let fs;
 
             beforeEach('create fakes', () => {
@@ -146,26 +138,66 @@ describe('TLS secure context utilities', () => {
                 secureContext = require('../../../lib/tls/secure-context');
             });
 
-            it('reads the PEM file content', () => {
+            it('creates a proper secure context after reading the PEM file', () => {
                 td.when(tlsVersions.allowed()).thenReturn([]);
                 td.when(tlsVersions.supported()).thenReturn([]);
                 td.when(tlsCiphers.defaults()).thenReturn([]);
                 td.when(fs.readFileSync('foo')).thenReturn('bar');
 
-                return expect(secureContext.create({ ca: 'foo' })).to.deep.include({ ca: 'bar' });
-            });
-
-            it('rejects unauthorized servers', () => {
-                td.when(tlsVersions.allowed()).thenReturn([]);
-                td.when(tlsVersions.supported()).thenReturn([]);
-                td.when(tlsCiphers.defaults()).thenReturn([]);
-                td.when(fs.readFileSync(), { ignoreExtraArgs: true }).thenReturn();
-
-                return expect(secureContext.create({ ca: 'foo' })).to.deep.include({ rejectUnauthorized: true });
+                return expect(secureContext.create({ ca: 'foo' })).to.deep.include({ ca: 'bar', rejectUnauthorized: true });
             });
         });
 
-        context('when a path to a CRL PEM file is provided', () => {
+        context('when the certificate authority PEM file pointer is provided', () => {
+            let fs, isValidPEM;
+
+            beforeEach('create fakes', () => {
+                fs = td.replace('fs');
+                isValidPEM = td.function();
+
+                td.replace('../../../lib/validator', { isValidPEM });
+
+                secureContext = require('../../../lib/tls/secure-context');
+            });
+
+            it('creates a proper secure context using the PEM file contents', () => {
+                td.when(tlsVersions.allowed()).thenReturn([]);
+                td.when(tlsVersions.supported()).thenReturn([]);
+                td.when(tlsCiphers.defaults()).thenReturn([]);
+                td.when(isValidPEM({ value: 'foo' })).thenReturn(true);
+
+                expect(secureContext.create({ ca: 'foo' })).to.deep.include({ ca: 'foo', rejectUnauthorized: true });
+
+                return expect(td.explain(fs.readFileSync).callCount).to.equal(0);
+            });
+        });
+
+        context('when the multiple certificate authority PEM file pointers are provided', () => {
+            let fs, isValidArray, isValidPEM;
+
+            beforeEach('create fakes', () => {
+                fs = td.replace('fs');
+                isValidArray = td.function();
+                isValidPEM = td.function();
+
+                td.replace('../../../lib/validator', { isValidArray, isValidPEM });
+
+                secureContext = require('../../../lib/tls/secure-context');
+            });
+
+            it('creates a proper secure context using the contents of all PEM files', () => {
+                td.when(tlsVersions.allowed()).thenReturn([]);
+                td.when(tlsVersions.supported()).thenReturn([]);
+                td.when(tlsCiphers.defaults()).thenReturn([]);
+                td.when(isValidArray({ value: ['foo', 'bar'], validator: isValidPEM })).thenReturn(true);
+
+                expect(secureContext.create({ ca: ['foo', 'bar'] })).to.deep.include({ ca: ['foo', 'bar'], rejectUnauthorized: true });
+
+                return expect(td.explain(fs.readFileSync).callCount).to.equal(0);
+            });
+        });
+
+        context('when a path to a certificate revocation list PEM file is provided', () => {
             let fs;
 
             beforeEach('create fakes', () => {
@@ -183,6 +215,55 @@ describe('TLS secure context utilities', () => {
                 return expect(secureContext.create({ crl: 'foo' })).to.deep.include({ crl: 'bar' });
             });
         });
+
+        context('when the certificate revocation list PEM file pointer is provided', () => {
+            let fs, isValidPEM;
+
+            beforeEach('create fakes', () => {
+                fs = td.replace('fs');
+                isValidPEM = td.function();
+
+                td.replace('../../../lib/validator', { isValidPEM });
+
+                secureContext = require('../../../lib/tls/secure-context');
+            });
+
+            it('creates a proper secure context using the PEM file contents', () => {
+                td.when(tlsVersions.allowed()).thenReturn([]);
+                td.when(tlsVersions.supported()).thenReturn([]);
+                td.when(tlsCiphers.defaults()).thenReturn([]);
+                td.when(isValidPEM({ value: 'foo' })).thenReturn(true);
+
+                expect(secureContext.create({ crl: 'foo' })).to.deep.include({ crl: 'foo' });
+
+                return expect(td.explain(fs.readFileSync).callCount).to.equal(0);
+            });
+        });
+
+        context('when the multiple certificate revocation list PEM file pointers are provided', () => {
+            let fs, isValidArray, isValidPEM;
+
+            beforeEach('create fakes', () => {
+                fs = td.replace('fs');
+                isValidArray = td.function();
+                isValidPEM = td.function();
+
+                td.replace('../../../lib/validator', { isValidArray, isValidPEM });
+
+                secureContext = require('../../../lib/tls/secure-context');
+            });
+
+            it('creates a proper secure context using the contents of all PEM files', () => {
+                td.when(tlsVersions.allowed()).thenReturn([]);
+                td.when(tlsVersions.supported()).thenReturn([]);
+                td.when(tlsCiphers.defaults()).thenReturn([]);
+                td.when(isValidArray({ value: ['foo', 'bar'], validator: isValidPEM })).thenReturn(true);
+
+                expect(secureContext.create({ crl: ['foo', 'bar'] })).to.deep.include({ crl: ['foo', 'bar'] });
+
+                return expect(td.explain(fs.readFileSync).callCount).to.equal(0);
+            });
+        });
     });
 
     context('validate()', () => {
@@ -195,7 +276,7 @@ describe('TLS secure context utilities', () => {
             return expect(() => secureContext.validate({ tls: { enabled: false }, ssl: true, sslOptions: { ca: 'foo' } })).to.throw(error);
         });
 
-        it('fails a path to the CA chain PEM file is not valid', () => {
+        it('fails when the certificate authority chain is not valid', () => {
             const error = errors.MESSAGES.ER_DEVAPI_BAD_TLS_CA_PATH;
 
             expect(() => secureContext.validate({ tls: { enabled: true, ca: null } })).to.throw(error);
@@ -212,7 +293,7 @@ describe('TLS secure context utilities', () => {
             return expect(() => secureContext.validate({ ssl: true, sslOptions: { ca: {} } })).to.throw(error);
         });
 
-        it('fails a path to the CA chain PEM file is not valid', () => {
+        it('fails when the certificate revocation list chain is not valid', () => {
             const error = errors.MESSAGES.ER_DEVAPI_BAD_TLS_CRL_PATH;
 
             expect(() => secureContext.validate({ tls: { enabled: true, crl: null } })).to.throw(error);
@@ -246,17 +327,35 @@ describe('TLS secure context utilities', () => {
             return expect(() => secureContext.validate({ ssl: true, tls: { versions: {} } })).to.throw(util.format(error, {}));
         });
 
-        it('fails when the list of TLS versions contains invalid values', () => {
+        it('fails when the list of TLS versions only contains invalid values', () => {
             td.when(tlsVersions.allowed()).thenReturn(['foo', 'bar']);
+            td.when(tlsVersions.unsupported()).thenReturn([]);
 
-            return expect(() => secureContext.validate({ tls: { versions: ['foo', 'bar', 'baz'] } })).to.throw(util.format(errors.MESSAGES.ER_DEVAPI_BAD_TLS_VERSION, 'baz', 'foo, bar'));
+            return expect(() => secureContext.validate({ tls: { versions: ['baz', 'qux'] } })).to.throw(util.format(errors.MESSAGES.ER_DEVAPI_BAD_TLS_VERSION, 'baz', 'foo, bar'));
+        });
+
+        it('fails when all the TLS versions in the list are insecure', () => {
+            td.when(tlsVersions.allowed()).thenReturn(['foo', 'bar']);
+            td.when(tlsVersions.unsupported()).thenReturn(['baz', 'qux']);
+
+            return expect(() => secureContext.validate({ tls: { versions: ['baz'] } })).to.throw(util.format(errors.MESSAGES.ER_DEVAPI_INSECURE_TLS_VERSIONS, 'baz', 'foo, bar'));
         });
 
         it('fails when none of the TLS versions in the list are supported', () => {
-            td.when(tlsVersions.allowed()).thenReturn(['foo', 'bar', 'baz', 'qux']);
-            td.when(tlsVersions.supported()).thenReturn(['foo', 'bar']);
+            td.when(tlsVersions.allowed()).thenReturn(['foo', 'bar']);
+            td.when(tlsVersions.supported()).thenReturn([]);
+            td.when(tlsVersions.unsupported()).thenReturn([]);
 
-            return expect(() => secureContext.validate({ tls: { versions: ['baz', 'qux'] } })).to.throw(errors.MESSAGES.ER_DEVAPI_NO_SUPPORTED_TLS_VERSION);
+            return expect(() => secureContext.validate({ tls: { versions: ['foo', 'bar'] } })).to.throw(errors.MESSAGES.ER_DEVAPI_NO_SUPPORTED_TLS_VERSION);
+        });
+
+        it('does not fail if any TLS version in the list if effectively supported by the client', () => {
+            td.when(tlsVersions.allowed()).thenReturn(['foo', 'bar']);
+            td.when(tlsVersions.supported()).thenReturn(['foo']);
+            td.when(tlsVersions.unsupported()).thenReturn(['baz']);
+            td.when(tlsCiphers.overlaps(), { ignoreExtraArgs: true }).thenReturn(['qux']);
+
+            return expect(secureContext.validate({ tls: { versions: ['foo', 'bar', 'baz'] } })).to.be.true;
         });
 
         it('fails when the list of ciphersuites is badly specified', () => {
@@ -265,6 +364,7 @@ describe('TLS secure context utilities', () => {
             // We need to allow and support at least one version.
             td.when(tlsVersions.allowed()).thenReturn(['foo']);
             td.when(tlsVersions.supported()).thenReturn(['foo']);
+            td.when(tlsVersions.unsupported()).thenReturn([]);
 
             expect(() => secureContext.validate({ tls: { enabled: true, ciphersuites: null } })).to.throw(util.format(error, null));
             expect(() => secureContext.validate({ ssl: true, tls: { ciphersuites: null } })).to.throw(util.format(error, null));
@@ -286,6 +386,7 @@ describe('TLS secure context utilities', () => {
             // We need to allow and support at least one version.
             td.when(tlsVersions.allowed()).thenReturn(['baz']);
             td.when(tlsVersions.supported()).thenReturn(['baz']);
+            td.when(tlsVersions.unsupported()).thenReturn([]);
 
             td.when(tlsCiphers.overlaps(ciphersuites)).thenReturn([]);
 
